# Konference p2d2 2020

6.2.2020

13\. ročník


## PostgreSQL 12

Tomáš Vondra

- Novinky pro administrátory
    - *SSL/TLS*
        - Nové sloupce pro (klientské?) certifikáty
        - Umožňuje specifikovat protokoly
            - TLS 1.0-1.3
            - Záleží na použité OpenSSL knihovně

    - *GSSAPI* šifrování
        - Umožňuje šifrování bez TLS vrstvy
        - Kerberos, ...?

    - `VACUUM`
        - `SKIP_LOCKED` možnost: přeskočí relace, které není možné vyčistit

    - Import
        - `COPY FROM` má nyní `WHERE` klauzuli

    - *CSV* výstupní formát
        - Možnost exportu do *csv* přímo v klientovi `psql`
        - `\pset format csv`


    - *pg_stat_statements*
        - Umožňuje resetovat statistiky pro jednotlivé dotazy
        - `SELECT pg_stat_statements_reset(queryid => X);`


    - *Progress monitoring*
        - V systémovém katalogu je možné získat stav (procenta) různých úloh
        - Nově se přidává *progress monitoring* pro
            - `CREATE INDEX`
            - `REINDEX`
            - `CLUSTER`
            - `VACUUM FULL`

    - *Checksums*
        - Od PostgreSQL 9.5
        - Nutné zvolit při vytváření *clusteru*
        - Od PostgreSQL 12 je možné zapnout/vypnout při vypnuté databázi


- Novinky pro vývojáře
    - *Generated* sloupce
        - Podobně jako trigger
        - Standardní sloupec, který je ukládaný
        - Při ukládání dat je vždy přepočítaný

    - *ENUM* typ
        - Umožňuje přidat hodnotu v transakci: podporuje ROLLBACK i COMMIT
        - Ve stejné transakci však nelze tuto hodnotu použít

    - *jsonpath* dotazování
        - Pro datový typ `jsonb`

    - Nový typ statistik *MCV*

- Replikace
    - `max_wal_senders` nyní není zahrnuto v `max_connections`, nyní je to
      dedikovaný parametr pro *wal sender* procesy

    - `recovery.conf`
        - Integrovany do `postgresql.conf`
        - Některé volby stačí aplikovat bez restartu serveru
        - Signalizažní soubory pro start replikace jsou
            - `recovery.signal`
            - `standby.signal`

- Partitioning
    - Flexibilnější rozdělení (podporuje *generalized expressions*)
    - Vylepšení práce se zámky při exekuci a rozšiřování


## JavaScript inside PostgreSQL

Simone Sanfratello


- Tools
    - PostgreSQL 11+
    - plv8 extension (extension with v8 JS engine)

- Usage:
    - Execute java script code inside PostgreSQL
        - Functions
        - Triggers
        - Procedures
    - Data types are mapped automatically

- Limitations
    - No modules, unless they're compiled within
    - SQL statements are dynamic, however checked at runtime, not during
      definition
    - No async, no event_loop
    - Possibility of SQL injection into the code


## Is JSONB a Silver Bullet

Angus Dippenaar

- JSON
    - Checks if the value is a valid JSON
    - Stored as text
- JSONB
    - Stored as binary data
    - Sorted and unified keys (removed duplicities)
    - Many operators
    - Indexes (and constraints?) on keys


## PostGIS v praxi

Michal Páleník

- Problém
    - Kontinenty se hýbají: centimetry za rok,
        - GPS s tímto nepracuje
        - Jiné standardy
    - Projekce: Jak zobrazit 3D povrch „koule” na 2D mapu
        - Krovak
            - Zachovává úhly
        - Mercator
            - Zachovává velikost
- Open Street Map
    - ODbL
- Import do DB
    - odbm2pg

- PostGIS
    - Geography
        - WGS84
    - Geometry
        - More functions, faster, strange distances (?)


## Zkušenosti získané laděním SQL dotazu a dopady na testování aplikace

Aleš Zelený

- Znát svá data!
- Použít `UNION ALL` místo `UNION`, kde nemůže dojít k duplicitám
    - Ušetří se řazení kvůli odstranění duplicit
    - Velký *system time* → často chyba v obnovování spojení (*connection
      pooler*)
- `pg_stat_statments` – ladění a metriky, frekvence pouštěných dotazů
- Označení funkce (PL/pgSQL i SQL)
    - Označen R/O funkce jako *STABLE* výrazně sníží počet jejího volání


## Professional PostgreSQL scheduling made easy

Pavlo Golub

- Yet another PG scheduler
- Current solutions
    - `pgagent`
    - `jpgagent`
    - ``

- `pg_timetable`
    - One binary, written in go
    - Runs with normal user privileges
    - No extras needed but to create a schema
    - Cross-platforms support


## Monitoring PostgreSQL

Roman Fišer

- Latence
    - Z `pg_stat_statements`
    - Průměrná a maximální doba vykonávání
- Chybovost
    - Rollback/Commit ratio
- Saturace
    - `num_backends`/`num_connections`
    - Checkpoints interval
    - Temp tables usage
    - CPU and IO
- Další
    - *idle in transaction*
    - Blokované *auto vacuum*
    - Dead lock

- Prometheus stack pro metriky
    - Prometheus
    - Alert manager
    - Grafana
        - Možné zobrazit i statická data
        - Nastavení: `pg_settings_*`
- ELK stack pro logy
    - RabbitMQ
    - Logstash
    - Elasticsearch
    - Kibana


## Profilování PL/pgSQL funkcí

Pavel Stěhule

- `plpgsql_check`
    - Top nástroj na profilování a ladění PL/pgSQL
- Pro dekompozici v PL/pgSQL používat pohledy, ne funkce
    - Funkce brání optimalizátoru SQL exekuce
- Konfigurace optimalizace
    - Ve výchozí konfiguraci PG neoptimalizuje dotazy, kde je více jak 8 JOINů
    - Planning time by byl signifikantní
    - Konfigurace `from_collapse_limit`, `join_collapse_limit`
- Využívání indexů
    - `pg_stat_user_indexes`
