# Základy PL/pgSQL

1.6.2022

Pavel Stěhule


## Motivace

- Odpadá potřeba stahovat data na klienta a pracovat s nimi lokálně
- Provedení kódu na DB serveru
- Zvyšuje možnost dekompozice – odstínění dat apod.
- Vysoká bezpečnost
    - Je možné vytvořit API k databázi a k datům, které nelze obejít
    - Řízení přístupu, operací a manipulace s daty přes uložené procedury,
      nikoliv v aplikaci

*Nevýhody*:
- Horší přenositelnost


## Procedury

- Nativní vs. externí jazyky
- Nativní pracují s SQL
    - PL/pgSQL, PL/SQL
    - SELECT je prvek jazyka, nikoliv „řetězec”
- Externí jazyky jsou v „cizím” jazyce
    - V Pythonu, Perlu, ...
    - V MSSQL např. C#
    - Většinou se volá externí interpret jazyka, pořád běží na serveru
- Nevhodné opouštět transakční svět DB serveru do netransakčního okolního světa (Internet)
    - „Untrusted” externí jazyky jsou velmi nebezpečné, dokáží shodit/nabourat
      správně chování procesů DB
- Procedury mají řešit pouze databázové operace a záležitosti
    - Nikoliv formátování a prezentaci dat


## PL/pgSQL

- Kompilovaný
- Interpret běží in-process uvnitř DB engine
- Funkce jsou uvnitř selectu, nemůžou pracovat s transakcemi
- Procedury se volají zvlášť, mimo select; Můžou pracovat s transakcemi
- Funkce i procedury uložené v relaci `pg_proc`
- Funkce mohou být přetížené
    - Jméno parametru je součástí signatury
- Kód procedur i funkcí je kód
    - Součást DB schematu
    - Soubory, verzování, reviews, testy, ...
    - Statická analýza: `plpgsql_check`

### Dynamické vs. statické

- *Statické SQL* – SQL výraz se nemění, pouze data či parametry
- *Dynamické SQL* – samotný SQL výraz, je dynamický (sestavuje se název tabulky,
  sloupce apod.)
    - Nedoporučuje se
    - Může být náchylné na SQL injection
    - Vstupní data nutné ošetřit: `quote_ident()`, `quote_literal()`,
        resp. `format()` s `%I`, `%L`
    - „EXECUTE”, `\gexec` ❗

### Flags

Pro optimalizátor

- `VOLATILE` – volá se pro každý select, pro každé porovnání (výchozí)
- `STABLE` – vrací stejnou hodnotu v rámci jednoho selectu resp. transakce
- `IMMUTABLE` – nijak nezávisí na stavu databáze; Se stejnými parametry vrátí
  vždy stejnou hodnotu


## Monitoring funkcí

- Nutné zapnout parametr `track_functions`
- Poté se budou sbírat data do pohledu `pg_stat_user_functions`


## Ladění

- Logovat dotazy trvající dele než *50 ms*
- Nástroje `pgFouine` a `pgBadger` na analýzu logu, pomalých dotazů
- Vícesloupcové indexy
- Generovaná garbage data odpovídající 3-5 let provozu
    - Pro testování výkonu
