# Docker školení

Ondřej Šika  
7\.-8\. října 2019  
[github](https://github.com/ondrejsika/docker-training)


## Příkazy

```
docker image ls -a
docker container ls -a
docker system df
```


### lokálně

```
docker info
```


### vzdáleně

client-server

```
DOCKER_HOST=ssh://docker.xmyslivec.cz docker info
```


### Pročištění dockeru

```
docker container prune
docker system prune
docker system prune --all
docker system prune --volumes
```


### Informace

```
docker info
docker history
docker inspect
```


### Spouštění

```
docker run hello-world
docker run debian
docker run -ti debian
docker run --rm debian echo test
docker run -ti --name pet debian
```


### Spouštění na pozadí

```
docker run -d debian ping 193.17.47.1
docker run -d --name ping debian ping 193.17.47.1
docker stop ping
docker start ping
docker stop ping
docker rm ping
```


### Logy

```
docker logs ping
docker logs -f ping
```

log pluginy (ELK, apod.)


### Proměnné prostředí

```
docker run -e TEST=fest debian env
docker run --env-file config.env debian env
docker run -e EDITOR debian env
```

Restart politiky
- `--restart on-failure`
- `--restart always` Zapne kontejner i po startu Dockeru
- `--restart unless-stopped` Restartuje kontejner dokud se nestopne Docker

### Vstup do běžícího kontejneru

```
docker run --name db -d postgres
docker exec -ti -u postgres db psql
docker exec -ti -u postgres db bash
```

### Volumes

Každý kontejner startuje se svým vytvořeným *volume* (adresářem...)

```
docker run -ti -v /data debian
docker run -ti -v my-volume:/data debian
```

```
docker run -d -v db-data:/var/lib/postgresql/data postgres
ls -la /var/lib/docker/volumes/db-data
```


### Security

```
docker run -ti -v /:/host debian
```


### Docker kontejner pro ovládání docker kontejnerů

```
docker run -d -v /var/run/docker.sock:/var/run/docker.sock --name ci docker
```

Docker kontejner `name` má přístup na docker socket na hostovi.


### Port forwarding

Neboli expose:
```
docker run -d -p 8080:80 nginx
```

Pouze z localhostu:
```
docker run -d -p 127.0.0.1:8080:80 nginx
```

Případně zjistit IP adresu:
```
docker inspect -f '{{.NetworkSettings.IPAddress}}' c2f253686e0b
```


## Dockerfile

Předpis sestavení image.

- dockerignore
    - Důležitá věc
    - Před buildem se zkopírují všechny soubory v aktuálním adresáři
    - Proto je vhodné zavést dockerignore a uvést nepotřebné (velké) soubory

- Vhodné pořadí pro optimalizaci a využití cache

- Možnost multistage sestavení
    - šetří místo hotových image

- Buildkit
    - Nová možnost pro sestavení
    - Zrychluje build
    - Zlepšuje cache


### Entry point

1. Příkaz píšeme jako pole
    ```
    CMD ["echo", "hello", "world"]
    ```

    Protože jinak pouští `sh -c 'CMD'` a PID 1 je shell a ne daný příkaz.

2. `CMD` můžeme rozdělit na *entrypoint* a *command*
    ```
    ENTRYPOINT ["echo"]
    CMD ["hello", "world"]
    ```

    Potom `run` spouští `ENTRYPOINT` s argumenty v `CMD` a případně `run něco`
    přepisuje pouze *argumenty*


### Pre-seed db

- image postgres má entrypoint, který umí spouštět `sh` skripty a `sql` soubory
- Stačí přidat `sql` soubor s vytvořením tabulek a vložením dat
- Po startu kontejneru pak budou data v databázi připravená


# Docker školení pro pokročilé


## Docker-compose

Instalace:
- Buď stáhnout "binárku"
- Přes pip


### Příkazy

```
docker-compose config
docker-compose build
docker-compose up -d
docker-compose down
docker-compose start
docker-compose stop
docker-compose ps
docker-compose logs
docker-compose exec service command
docker-compose push
```


### Proměnné prostředí

1. Přímo hodnoty v `docker-compose.yml`
2. Ze souboru

- Výchozí hodnoty shell syntaxí `${FOO:-bar}`
- Proměnné prostředí i pro ovládání docker a docker-compose
    - Např. `DOCKER_HOST` nebo `COMPOSE_PROJECT_NAME`


### Demo

viz `demo-compose-2/`


### Více souborový compose

Např. pro oddělení produkčního a vývojového prostředí.


## Docker machine

Nástroj pro správu Docker hostů pro běh kontejnerů. Provisioning virtuálů pro
Docker host, původně připravený pro běh Dockeru na Windows.


### Příkazy

```
docker-machine ls
docker-machine env
docker-machine ip worker1
docker-machine ssh worker1 command args
docker-machine create --driver digitalocean ci
```



## Docker swarm

Nativní docker cluster. Ne tak mocný jako Kubernetes.


### Sestavení clusteru

```
docker swarm init --advertise-addr <manager_ip>
docker swarm join --token <token> <manager_ip>:2377
docker node ls
```


### Demo

viz `demo-compose-2/`


### Provoz produkce

- Databáze mimo docker swarm
- Storage: ceph, nfs nebo S3
    - Pro S3 je mozne provozovat `minio` -- S3 na nedistribuovaném storage
