# Kubernetes školení

Ondřej Šika  
20\.-21. únor 2023  

- [web](https://ondrej-sika.cz/skoleni/kubernetes/)
- [prezentace](https://sika.link/k8s-slides)
- [github](https://github.com/ondrejsika/kubernetes-training)


## Co je Kubernetes

- Abstrakce od hardware
    - V cloudu možnost serverless nodů
- Deklarativnější přístup
    - Neumí automatické mazání jako např. Terraform
    - Úpravy/změny se nasazují/aplikují přirozeně
- Automatizuje velkou sadu manuálních úkolů
    - Restart zaseklých podů, ...
    - Přeplánování aplikací při výpadku nodu
    - ...
- Nový standard pro provozování aplikací
    - Některé nástroje jsou nově připravovány rovnou pro nasazení do Kubernetes

> Docker Swarm a Kubernetes byli kdysi konkurenční technologie. Nyní je Swarm
> mrtvý a Kubernetes standard.


## Architektura

- 2 typy nodů
    - Control plane nodes
        - `etcd`
        - Controller manager
        - API server
    - Worker nodes
        - `kubelet` (mluví s API serverem)
        - `kubeproxy` (routuje provoz)
        - Pods


## Příkazy

- [kube-ps1](https://github.com/jonmosco/kube-ps1)
- Zkratky v API
    - Např. `nodes`/`node`/`no`
- proxy
    - `kubectl proxy`


- Kubernetes dashboard
    - Pro minikube i pro jiné poskytovatele
    - [github deployment](https://github.com/kubernetes/dashboard)
    - `minikube addons enable dashboard`
    - Vhodné povolit i metriky – též pro `k top`
        - `minikube addons enable metrics-server`

- **k9s**
    - k8s dashboard v CLI
    - *MUST HAVE!*
    - [k9cli.io](https://k9scli.io/)


## Pod

- Sada kontejnerů
- Nejmenší spustitelná jednotka v Kubernetes
- Sdílí síťový namespace
    - Mají jednu adresu
    - Nemůže více kontejnerů poslouchat na stejném portu
- Pod != kontejner
    - Typicky pro nginx+fpm (PHP)
- *Resources*
    - `requests`
        - Informace pro plánovač
        - Udává, kolik bežně pod vyčerpá
        - Plánovač nenastartuje pod, pokud se nikam nevejde
    - `limits`
        - Informace pro běh kontejnerů
        - Pokud pod přesáhne limit CPU, worker pod omezí
        - Pokud přesáhne RAM, nedostane ji → nejspíš spadne


## Přiřazení podů na nody

- Mapování Podů na konkrétní nody
    - Lépe na skupinu nodů pomocí labels
- Kvůli lepšímu škálování
    - Pody náročné na paměť vs. pody náročné na CPU
- Afinita
    - Afinita a anti-afinita
    - Podrobnější pravidla
    - I např. vztahy mezi pody a preference (fuzzy)


## Replica set

- Šablona podu(ů)
- Udržuje více replik určitého podu
- Z historických důvodů vyžaduje propojení přes `selector.MatchLabels` a `labels` u podu
- Service pak náhodně směřuje na jednotlivé pody v replica setu


## Deployment

- Šablona replica setu
- Především kvůli updatům/změnám v replica setu
- Navíc „verzuje” replica sety
    - Kvůli rychlému rollbacku/undo
    - `kubectl history`, `kubectl rollout history deployment...`
    - Nicméně tuto funkcionalitu nahrazuje git, helm, ArgoCD
    - `revisionHistoryLimit` je může vypnout

- Graceful restart
    - `kubectl rollout restart`


### Deployment strategy

- *Recreate*
    - Všechno vypne, pak všechno zapne
    - Pro nekompatibilní databázové migrace
    - Data ve frontách (navíc nevadí výpadek)
- *Rolling update*
    - Pro postupný upgrade
    - Nutno mít možnost běžet simultánně starou a novou verzi
    - Parametry pro kolik podů najednou, kolik musí být up...
- *Blue-green update*, *Cannary*, ...
    - Není nativně v k8s
    - Jako feature flags, ale na úrovni deploymentu
    - Umí např. ArgoCD


## Stateful sets

- Nepouštět, pokud není nutné
- Vytáčet managované databáze jinde – v cloudu

### Service discovery

- Speciální `service` resource s parametrem `clusterIP: None`
- „Prozradí” IP adresy všech podů
    - Nastaví na jeden hostname více IP adres - adres podů
    - Na rozdíl od běžné *service* nedělá „rozhraní” ani port-mapping
- Pro propojení „replik” mezi sebou


## Daemon sets

- Spustí jednu repliku (pod) na každém nodu clusteru
- Typicky pro servisní úkony, ne pro aplikaci
    - Sběr metrik, sběr logů


## Jobs

- Objekty pro jednorázové úlohy
- Např. CI build, databázová migrace, ...
    - Časový limit `activeDeadlineSeconds`, nezapomenout na `restartPolicy`
- Můžou se pouštět paralelně (stres test, ...)
    - Pokročilejší plánování v ArgoCD
- Plánovač – cron jobs
    - Pěkné vlastnosti: `concurrencyPolicy`, `restartPolicy`
    - [crontab.guru](https://crontab.guru/)


## Přístup zvenku

- Možnosti
    - Service s node portem
    - Service load balancer
    - Ingress

### Service node port

- Vystavení portu service na nodu
    - Povolený rozsah portů 30000-32767
- Otevřou se na všech nodech v clusteru
    - `externalTrafficPolicy` ovlivní toto chování (lokální routing místo mesh, 
- Typicky namířit externí load balancer na tyto porty

### Service load balancer

- Vystaví externí službu (load balancer v cloudu) a nasměruje traffic na pody
- *Síťový balancer*
- Existují i varianty, že vystaví balancer v rámci clusteru
    - Není 100% HA, chová se spíše jako Virtual-IP

### Ingress

- Pravidla pro reverzní proxy – aplikační load balancer
- Řeší Ingress controller, který je třeba doinstalovat

- Potřeba Service typu Load balancer nebo mít připravený balancer (Terraformem)
    - DNS řešit pomocí ExternalDNS není zcela vhodné kvůli možnému úniku
      přístupům k DNS – potencionálně velký problém

**Certifikáty**:
- `certManager`
- Nastavení v resource `ClusterIssuer`
    - Provázání potom na ingress pravidla pomocí annotations


## Init kontejnery

- Součást podu
- Všechny init kontejnery musí doběhnout
- Hodí se pro setup
    - Počkat, než doběhnou migrace nebo nastartuje databáze
    - Vygenerovat startup data
    - ...
    - `wait-for-it.sh`


## Namespace

- Málokdy se používá namespace v souborech
- Přes příkazovou řádku nebo v konfiguraci
- Nástroj `kubens` (a `kubectx`)

### Resource kvóty

- Kvóty na namesapce `ResourceQuota`
- Dáva smysl omezit počet podů, vypnout load balancery (stojí 💲)



## Storage

- Persistent Volume Claims z historických důvodů
    - Persistent volumes byly manuálně, staticky
    - PVC dynamicky přiřazovaly PV
    - Nyní obojí dynamicky
- StorageClass
    - Výchozí bývá s `RetainPolicy: delete` – tedy že smaže dynamicky vytvořený
      PV po smazání PVC
- File system vs. block device
    - Block device pouze v jednom podu
    - FS může být mountovaný do více podů zároveň, ale špatný výkon

- ⚠️ Pozor, pře HostPath je možné namountovat root nodu do podu
    - Omezení `AdmissionConfiguration` (Pod Security Policy)

### Longhorn

- Kubernetes native storage
- Umí file system i block device
- Využívá iSCSI targets
- Instalace přes Helm nebo ArgoCD


## Konfigurace

- Secrets a ConfigMap
    - Obojí je de facto shodné, i secrets jdou vypsat hodnoty
- Z historických důvodů
    - Secrets výchozí binární
    - ConfigMap výchozí textové


## Probes

- startup, liveness, readiness
- Vhodné definovat `startupProbe`
    - Nejvíc se ocení při updatech
- `livenessProbe` jako healthcheck endpoint


## Helm

- Repozitáře a balíčky pro nasazení aplikací do kubernetes
    - Tzv. Helm charts

- Helm má v balíčku šablony (Go template)
    - Předpis pro Kubernetes manifesty
    - Může mít parametry – *values*
- All in one příkaz
    - `helm upgrade --install`
    - Verzuje revize, lze jednoduše provést rollback
        - `helm rollback <release> <revision>`

### Vytvoření balíčku

- Prázdná šablona pomocí příkazu `helm create <package>`
    - Hromada zbytečných souborů a složitých šablon
- Možnost instalace
    - Ze složky
    - Z gitu
    - Sestavit balíček
- Možnost instalace
    - Ze složky
    - Z gitu
    - Sestavit balíček
        - `git package ...`
- Repozitář
    - Adresář s indexem a tarbally
    - Servírovaný přes web (stačí raw soubory github/GitLab repa)
    - `helm repo`


## TODO

⚠️  *NSA Kubernetes hardening guide* ⚠️

- RKE
    - Neomezuje počet podů
    - Může sdílet control plane s worker nody


## Otázky

- Best practice pro secrets s docker registry loginem
- Jak strukturovat manifesty
    - Co zdroj, to soubor
    - Podadresáře?
- Security, řízení přístupu
    - V ukázkách může všechno všude (pokud znám název nebo IP)
    - Stačí namespace?
        - Ne – service jsou přistupné napříč namespace
- Certifikaty cert-manager
